#!/usr/bin/python

import crypt
import Pyro.core
import time
import os, sys, shutil
from os import path

#import importerConf as opts
# Leo la config
if len(sys.argv) != 2:
    print >>sys.stderr, 'Usage: %s <configFile>\n' % sys.argv[0]
    sys.exit(1)

class cfg:
    pass
execfile( sys.argv[1], cfg.__dict__, cfg.__dict__ )
        
class QueueServer(Pyro.core.ObjBase):
    def __init__(self, type='fifo', dir='.', validUsers={}):
        Pyro.core.ObjBase.__init__(self)
        self.type = type
        self.dir = dir
        self.dirProcessed = path.join(self.dir,'processed')
        if not path.isdir(self.dirProcessed):
            try:#crear el directorio de procesados si no existe
                os.mkdir(self.dirProcessed)
            except OSError:
                print 'No se pudo crear el directorio para almacenar los mensajes procesados'
                sys.exit(1)
        self.validUsers = validUsers
        
    def _authenticate(self, loginInfo):
        '''
        Autenticar un usuario con su loginInfo (usuario, password)
        '''
        try:
            user = loginInfo[0]
            passwd = loginInfo[1]
        except IndexError:
            return False
        
        try:
            if self.validUsers[user] == crypt.crypt(passwd, passwd[:3]):
                return True
            else:
                return False
        except (IndexError, KeyError):
            return False

    
    def pop(self, loginInfo):
        '''
        Devolver un item del queue y borrarlo
        '''
        if not self._authenticate(loginInfo):
            return None
        
        i = self._nextItem(loginInfo)
        
        if not i == None:
            shutil.copyfile( path.join( self.dir, i[0] ), path.join(self.dirProcessed, i[0]))
            os.unlink(path.join(self.dir, i[0]))
            return i[1]
        else:
            return None
    
    def peek(self, loginInfo):
        if not self._authenticate(loginInfo):
            return None
        
        i = self._nextItem(loginInfo)
        if not i == None:
            return i[1]
        else:
            return None
    
    def _nextItem(self, loginInfo):
        '''
        Devolver un tuple (nombreDeArchivo, diccionario) del proximo item
        '''
        if not self._authenticate(loginInfo):
            return None
        
        files = []
        for fileName in os.listdir(self.dir):
            try: #no incluir los nombres de archivos que no son numericos
                float(fileName)
                files.append(fileName)
            except ValueError:
                continue

        files.sort(lambda x,y: cmp(float(x),float(y)))#Organizar los nombres de archivos de menor a mayor
        
        try:
            if self.type == 'fifo':
                filename = files[0]
            else:
                filename = files[-1]
        except IndexError:
            return None
        
        f = file(os.path.join( self.dir, filename ), 'r')
        txt = f.read()
        f.close()
        
        try: #Si no es codigo python valido, entonces devolverlo como string
            d = eval(txt)
        except SyntaxError:
            d = txt

        return (filename, d)
        
    def push(self, item, loginInfo):
        '''
        Agregar un item al queue, recibe un diccionario, y lo guarda como
        string en un archivo cuyo nombre es el epoch en el que se crea
        '''
        if not self._authenticate(loginInfo):
            return None
        
        filename = str(time.time())
        while path.isfile(path.join(self.dir,filename)): # Si el filename ya existe, agregar un 0 en la parte decimal
            filename = '%s0' % filename

        f = file(path.join(self.dir,filename), 'w')
        try:
            import pprint
            pprint.pprint( item, f )
        except:
            f.write(str(item))
        f.close()
        return True
        
if __name__ == '__main__':
    Pyro.core.initServer()
    q = Pyro.core.Daemon(port=cfg.pyroPort)
    uri=q.connect(QueueServer(type=cfg.type, dir=cfg.dir, validUsers=cfg.users),"queueServer")
    
    q.requestLoop()
